@@ -926,7 +926,7 @@ cfs_read_bz2 (struct CompressedFileSource *cfs,
 	  return rc;
 	}
       cfs->bstrm.next_in = buf;
-      cfs->bstrm.avail_in = (uInt) in;
+      cfs->bstrm.avail_in = (unsigned int) in;
       cfs->bstrm.next_out = cfs->result;
       cfs->bstrm.avail_out = COM_CHUNK_SIZE;
       cfs->result_pos = 0;
@@ -1190,38 +1190,42 @@ EXTRACTOR_datasource_create_from_file_ (const char *filename,
   struct EXTRACTOR_Datasource *ds;
   enum ExtractorCompressionType ct;
   int fd;
+#if WINDOWS
+  struct _stat sb;
+#else
   struct stat sb;
+#endif
   int64_t fsize;
   int winmode = 0;
 #if WINDOWS
   winmode = O_BINARY;
 #endif
 
-  if (-1 == (fd = open (filename, O_RDONLY | O_LARGEFILE | winmode)))
+  if (-1 == (fd = OPEN (filename, O_RDONLY | O_LARGEFILE | winmode)))
     {
       LOG_STRERROR_FILE ("open", filename);
       return NULL;
     }
-  if ( (0 != fstat (fd, &sb)) ||
+  if ( (0 != FSTAT (fd, &sb)) ||
        (S_ISDIR (sb.st_mode)) )       
     {
       if (! S_ISDIR (sb.st_mode))
 	LOG_STRERROR_FILE ("fstat", filename);
       else
 	LOG ("Skipping directory `%s'\n", filename);
-      (void) close (fd);
+      (void) CLOSE (fd);
       return NULL;
     }
   fsize = (int64_t) sb.st_size;
   if (0 == fsize)
     {
-      (void) close (fd);
+      (void) CLOSE (fd);
       return NULL;
     }
   bfds = bfds_new (NULL, fd, fsize);
   if (NULL == bfds)
     {
-      (void) close (fd);
+      (void) CLOSE (fd);
       return NULL;
     }
   if (NULL == (ds = malloc (sizeof (struct EXTRACTOR_Datasource))))
@@ -1243,7 +1247,7 @@ EXTRACTOR_datasource_create_from_file_ (const char *filename,
 	  LOG ("Failed to initialize decompressor\n");
 	  bfds_delete (bfds);
 	  free (ds);
-	  (void) close (fd);
+	  (void) CLOSE (fd);
 	  return NULL;
 	}
     }
@@ -1314,7 +1318,7 @@ EXTRACTOR_datasource_destroy_ (struct EXTRACTOR_Datasource *ds)
     cfs_destroy (ds->cfs);
   bfds_delete (ds->bfds);
   if (-1 != ds->fd)
-    (void) close (ds->fd);
+    (void) CLOSE (ds->fd);
   free (ds);
 }
 
