@@ -72,9 +72,9 @@ USA.  */
 
 #ifdef VMS
 #include <unixlib.h>
-#if HAVE_STRING_H - 0
-#include <string.h>
 #endif
+#if HAVE_STRING_H
+#include <string.h>
 #endif
 
 #if defined (WIN32) && !defined (__CYGWIN32__)
@@ -234,6 +234,9 @@ my_index (str, chr)
    and has done so at least since version 2.4.5. -- rms.  */
 extern int strlen (const char *);
 #endif /* not __STDC__ */
+#if defined(__APPLE__)
+extern size_t strlen (const char *);
+#endif
 #endif /* __GNUC__ */
 
 #endif /* not __GNU_LIBRARY__ */
