@@ -29,6 +29,7 @@
 #include <gst/pbutils/pbutils.h>
 #include <gst/tag/tag.h>
 #include <gst/app/gstappsrc.h>
+#include <pthread.h>
 
 GST_DEBUG_CATEGORY_STATIC (gstreamer_extractor);
 #define GST_CAT_DEFAULT gstreamer_extractor
@@ -47,6 +48,8 @@ GST_DEBUG_CATEGORY_STATIC (gstreamer_extractor);
  */
 #define DATA_TIMEOUT 80000LL /* 80ms */
 
+pthread_mutex_t pipe_mutex;
+
 /**
  * Struct mapping GSTREAMER tags to LE tags.
  */
@@ -951,6 +954,7 @@ feed_data (GstElement * appsrc,
 
   accumulated = 0;
   data_len = 1;
+  pthread_mutex_lock (&pipe_mutex);
   while ( (accumulated < size) && (data_len > 0) )
   {
     data_len = ps->ec->read (ps->ec->cls, (void **) &le_data, size - accumulated);
@@ -960,6 +964,7 @@ feed_data (GstElement * appsrc,
       accumulated += data_len;
     }
   }
+  pthread_mutex_unlock (&pipe_mutex);
   gst_memory_unmap (mem, &mi);
   if (size == accumulated)
   {
@@ -1002,7 +1007,9 @@ seek_data (GstElement * appsrc,
 	   struct PrivStruct * ps)
 {
   GST_DEBUG ("seek to offset %" G_GUINT64_FORMAT, position);
+  pthread_mutex_lock (&pipe_mutex);
   ps->offset = ps->ec->seek (ps->ec->cls, position, SEEK_SET);
+  pthread_mutex_unlock (&pipe_mutex);
   ps->last_data_request_time = g_get_monotonic_time ();
   return ps->offset == position;
 }
@@ -1024,7 +1031,6 @@ send_structure_foreach (GQuark field_id,
   struct PrivStruct *ps = user_data;
   gchar *str;
   const gchar *field_name = g_quark_to_string (field_id);
-  const gchar *type_name = g_type_name (G_VALUE_TYPE (value));
   GType gst_fraction = GST_TYPE_FRACTION;
   GQuark *quark;
 
@@ -1071,7 +1077,6 @@ send_structure_foreach (GQuark field_id,
     /* This is a potential source of invalid characters */
     /* And it also might attempt to serialize binary data - such as images. */
     str = gst_value_serialize (value);
-    g_print ("Will not try to serialize structure field %s (%s) = %s\n", field_name, type_name, str);
     g_free (str);
     str = NULL;
     break;
@@ -1392,8 +1397,9 @@ send_tag_foreach (const GstTagList * tags,
 
           mime_type = gst_structure_get_name (gst_caps_get_structure (caps, 0));
           info = gst_sample_get_info (sample);
-
-          if (!gst_structure_get (info, "image-type", GST_TYPE_TAG_IMAGE_TYPE, &imagetype, NULL))
+	  
+          if ( (NULL == info) ||
+	       (!gst_structure_get (info, "image-type", GST_TYPE_TAG_IMAGE_TYPE, &imagetype, NULL)) )
             le_type = EXTRACTOR_METATYPE_PICTURE;
           else
           {
@@ -1701,11 +1707,9 @@ send_toc_tags_foreach (const GstTagList * tags,
   struct PrivStruct *ps = user_data;
   GValue val = { 0 };
   gchar *topen, *str, *tclose;
-  const gchar *type_name;
   GType gst_fraction = GST_TYPE_FRACTION;
 
   gst_tag_list_copy_value (&val, tags, tag);
-  type_name = g_type_name (G_VALUE_TYPE (&val));
   switch (G_VALUE_TYPE (&val))
   {
   case G_TYPE_STRING:
@@ -1725,7 +1729,6 @@ send_toc_tags_foreach (const GstTagList * tags,
     /* This is a potential source of invalid characters */
     /* And it also might attempt to serialize binary data - such as images. */
     str = gst_value_serialize (&val);
-    g_print ("Will not try to serialize tag %s (%s) = %s\n", tag, type_name, str);
     g_free (str);
     str = NULL;
     break;
@@ -1918,7 +1921,9 @@ send_discovered_info (GstDiscovererInfo * info,
     case GST_DISCOVERER_MISSING_PLUGINS:
       break;
     }
+  pthread_mutex_lock (&pipe_mutex);
   send_info (info, ps);
+  pthread_mutex_unlock (&pipe_mutex);
 }
 
 
@@ -2031,7 +2036,6 @@ EXTRACTOR_gstreamer_extract_method (struct EXTRACTOR_ExtractContext *ec)
   ps.dc = gst_discoverer_new (8 * GST_SECOND, &err);
   if (NULL == ps.dc) 
     {
-      g_print ("Error initializing: %s\n", err->message);
       if (NULL != err)
 	g_error_free (err);
       return;
@@ -2087,6 +2091,8 @@ gstreamer_init ()
   subtitle_quarks[1] = g_quark_from_string (NULL);
 
   duration_quark = g_quark_from_string ("duration");
+
+  pthread_mutex_init (&pipe_mutex, NULL);
 }
 
 
